From e3d420ee3ce896e8262ba1821ae028c1c5dc57e4 Mon Sep 17 00:00:00 2001
From: Liang Zhang <zhangliang5@huawei.com>
Date: Mon, 1 Mar 2021 15:20:11 +0800
Subject: [PATCH 3/4] device_model: interrupt guest when tap send finished

Signed-off-by: Liang Zhang <zhangliang5@huawei.com>
---
 device_model/src/virtio/net.rs | 11 +++++++++++
 1 file changed, 11 insertions(+)

diff --git a/device_model/src/virtio/net.rs b/device_model/src/virtio/net.rs
index 658603e..075b165 100644
--- a/device_model/src/virtio/net.rs
+++ b/device_model/src/virtio/net.rs
@@ -272,6 +272,7 @@ impl NetIoHandler {
 
     fn handle_tx(&mut self) -> Result<()> {
         let mut queue = self.tx.queue.lock().unwrap();
+        let mut need_irq = false;
 
         while let Ok(elem) = queue.vring.pop_avail(&self.mem_space, self.driver_features) {
             let mut read_count = 0;
@@ -299,6 +300,16 @@ impl NetIoHandler {
                 .vring
                 .add_used(&self.mem_space, elem.index, 0)
                 .chain_err(|| format!("Net tx: Failed to add used ring {}", elem.index))?;
+
+            need_irq = true;
+        }
+
+        if need_irq {
+            self.interrupt_status
+                .fetch_or(VIRTIO_MMIO_INT_VRING, Ordering::SeqCst);
+            self.interrupt_evt
+                .write(1)
+                .chain_err(|| ErrorKind::EventFdWrite)?;
         }
 
         Ok(())
-- 
2.27.0

